WRITING_STYLES = [
    "narrative",
    "expository",
    "descriptive",
    "persuasive",
    "technical",
    "poetic",
    "journalistic",
    "academic",
    "conversational",
    "formal",
    "informal",
    "humorous",
    "dramatic",
    "satirical",
    "romantic",
    "suspenseful",
    "reflective",
    "critical",
    "analytical",
    "speculative",
    "historical",
    "scientific",
    "philosophical",
    "biographical",
    "autobiographical",
    "epistolary",
    "didactic",             
    "stream-of-consciousness", 
    "ironic",
    "colloquial",          
    "aphoristic",          
    "allegorical",
    "minimalist",
    "baroque",              
    "metafictional",
    "emotive",
]


MEDIUMS = [
    "novel",
    "short story",
    "poem",
    "essay",
    "article",
    "blog post",
    "speech",
    "letter",
    "report",
    "review",
    "memoir",
    "journal entry",
    "script",
    "play",
    "song lyrics",
    "advertisement",
    "forum post",
    "tweet",
    "social media comment",
    "podcast transcript",
    "text message",
    "dialogue",
    "instruction manual",
    "editorial",
    "press release",
    "whitepaper",
]


PERSONAS = [
    "first-person narrator",
    "third-person omniscient narrator",
    "unreliable narrator",
    "character in a story",
    "historical figure",
    "scientist",
    "teacher",
    "student",
    "journalist",
    "blogger",
    "critic",
    "poet",
    "novelist",
    "playwright",
    "advertiser",
    "AI assistant",
    "customer",
    "marketer",
    "influencer",
    "parent",
    "child",
    "elder",
    "comedian",
    "philosopher",
    "entrepreneur",
]


INTENTIONS = [
    "inform",
    "entertain",
    "persuade",
    "express opinion",
    "complain",
    "praise",
    "reflect",
    "warn",
    "recommend",
    "summarize",
    "neutral",
    "angry",
    "joyful",
    "sarcastic",
    "hopeful",
    "fearful",
    "melancholic",
    "enthusiastic",
    "cynical",
    "nostalgic",
]


SENTENCE_LENGTH_OPTIONS = [
    "A couple of words",
    "Unfinished sentence",
    "1 short phrase",
    "1 short sentence", 
    "1 very very short sentence",
    "1 sentence", 
    "2 sentences", 
    "Short Paragraph", 
    "Long Paragraph"
]


SENTIMENT_LABEL_MAPPING = {
    2: ["positive", "negative"],
    3: ["positive", "neutral", "negative"],
    5: ["1 star", "2 stars", "3 stars", "4 stars", "5 stars"]
}

